import React, {Fragment, Component} from 'react';
import PropTypes from 'prop-types';

import {withStyles} from '@material-ui/core/styles';
import Grid from '@material-ui/core/Grid';

import Select from 'react-select';
import {DateTimePicker} from 'material-ui-pickers';

import {Query, Mutation} from 'react-apollo';
import gql from 'graphql-tag';

import Progress from './progress';
import selectComponents, {selectStyles} from './autocompleteSelect';

import {capitalize} from 'lodash';
import Error from "./error";


class AgentsControls extends Component {
    getControlValuesQuery = gql`
        query {
          channels:agent(distinct_on:channel) {
            channel
          }
          teams:agent(distinct_on:team) {
            team
          }
          sites:agent(distinct_on:site) {
            site
          }
          tiers:agent(distinct_on:tier) {
            tier
          }
          selectedStartDate @client
          selectedEndDate @client
          selectedChannels @client
          selectedTiers @client
          selectedSites @client
          selectedTeams @client
        }
    `;

    setAgentControlsQuery = gql`
        mutation setAgentControls($startDate: Int!, $endDate: Int!,
          $c: [String]!, $ts: [String]!, $s: [String]!, $tr: [String]!) {
          setAgentControls(startDate: $startDate, endDate: $endDate, channels: $c,
            teams: $ts, sites: $s, tiers: $tr) @client
        }
    `;

    parseControlsData(data, type) {
        return data[type + 's'].map(x => ({
            value: x[type],
            label: x[type],
        }));
    }

    renderSelector(data, type, onChange) {
        const {classes, selectStyles} = this.props;
        return (
            <Select
                isMulti
                options={this.parseControlsData(data, type)}
                classes={classes}
                components={selectComponents}
                textFieldProps={{
                    label: capitalize(type),
                    InputLabelProps: {
                        shrink: true,
                    },
                }}
                styles={selectStyles}
                onChange={onChange}
            />
        )
    }

    render() {
        const {classes} = this.props;
        return (
            <div className={classes.controlWrapper}>
                <form className={classes.root} autoComplete='on'>
                    <Grid container spacing={16}>
                        <Query query={this.getControlValuesQuery}>
                            {({loading, error, data}) => {
                                if (loading) return <Progress/>;
                                if (error) return <Error msg={'Failed to load controls'} />;

                                return (
                                    <Mutation mutation={this.setAgentControlsQuery}>
                                        {setAgentControls => {
                                            if (loading) return <Progress/>;
                                            if (error) return <Error msg={'Failed to set local settings'} />;

                                            return (
                                                <Fragment>
                                                    <Grid item sm={12} md={6} lg={1}>
                                                        <DateTimePicker
                                                            ampm={false}
                                                            value={data.selectedStartDate}
                                                            onChange={x => {
                                                                setAgentControls({
                                                                    variables: {startDate: x.utc().valueOf()}
                                                                });
                                                            }}
                                                            label='From'
                                                            showTodayButton
                                                            className={classes.agentsControlDateTimePicker}
                                                        />
                                                    </Grid>
                                                    <Grid item sm={12} md={6} lg={1}>
                                                        <DateTimePicker
                                                            ampm={false}
                                                            value={data.selectedEndDate}
                                                            onChange={x => {
                                                                setAgentControls({
                                                                    variables: {endDate: x.utc().valueOf()}
                                                                });
                                                            }}
                                                            label='To'
                                                            className={classes.agentsControlDateTimePicker}
                                                        />
                                                    </Grid>
                                                    <Grid item sm={12} md={8} lg={3}>
                                                        {this.renderSelector(data, 'channel', x => {
                                                            setAgentControls({
                                                                variables: {c: x.map(y => y.value)}
                                                            });
                                                        })}
                                                    </Grid>
                                                    <Grid item sm={12} md={4} lg={2}>
                                                        {this.renderSelector(data, 'tier', x => {
                                                            setAgentControls({
                                                                variables: {tr: x.map(y => y.value)}
                                                            });
                                                        })}
                                                    </Grid>
                                                    <Grid item sm={12} md={6} lg={3}>
                                                        {this.renderSelector(data, 'site', x => {
                                                            setAgentControls({
                                                                variables: {s: x.map(y => y.value)}
                                                            });
                                                        })}
                                                    </Grid>
                                                    <Grid item sm={12} md={6} lg={2}>
                                                        {this.renderSelector(data, 'team', x => {
                                                            setAgentControls({
                                                                variables: {ts: x.map(y => y.value)}
                                                            });
                                                        })}
                                                    </Grid>
                                                </Fragment>
                                            )
                                        }}
                                    </Mutation>
                                )
                            }}
                        </Query>
                    </Grid>
                </form>
            </div>
        );
    }
}

AgentsControls.propTypes = {
    classes: PropTypes.object.isRequired,
    selectStyles: PropTypes.object.isRequired,
};

export default withStyles(selectStyles)(AgentsControls);