import React, {Component} from 'react';
import moment from 'moment';
import {Query, withApollo} from 'react-apollo';
import gql from 'graphql-tag';

import AgentDataTable from './table';
import {updateColumnOptions, updateToStringMethod} from './table';
import Progress from './progress';
import AgentsTableToolbar from './agentsTableToolbar';
import TableColumnSelect from './tableColumnSelect';
import Error from './error';


class AgentsTable extends Component {

    getAgentsQuery = (agentTable) => gql`
        query($start: timestamp!, $end: timestamp!, $channel: [String]!, 
          $team: [String]!, $tier: [String]!, $site: [String]!) {
          
          updatedAgentsTableQueues @client {
            agents {
              agentId
              activeQueues @client
            }
          }
          
          agents: ${agentTable} (order_by: {start_time_utc: asc}, where: {
            _or: [
              { start_time_utc: {_gte: $start _lte: $end} }
              {
                start_time_utc: {_lte: $start}
                end_time_utc: {_gte: $start}
              }
            ],
            agent: {
              channel: {_in: $channel}
              team: {_in: $team}
              tier: {_in: $tier}
              site: {_in: $site}
            }
          }) {
            id
            start_time_utc
            end_time_utc
            ${agentTable === 'agent_off_shift' ? 'off_reason' : ''}
            shift {
              id
              name
            }
            agent {
              id
              first_name
              last_name
              email
              channel
              site
              position
              position_abbr
              tier
              team
              active_queues: agent_shift_queues {
                queue {
                  id
                  name
                }
              }
              assigned_queues: agent_assigned_queues {
                queue {
                  id
                  name
                }
              }
              team_manager: agent_team_manager {
                id
                email
              }
            }
          }
        }
    `;

    getAgentControlsQuery = gql`
        query {
          selectedStartDate @client
          selectedEndDate @client
          selectedChannels @client
          selectedTiers @client
          selectedSites @client
          selectedTeams @client
          isOnShift @client
          agentsTableColumnsOptions @client {
            name
            display
          }
        }
    `;

    columns = [
        {
            name: "agent_id",
            label: "Agent ID",
            options: {
                filter: false,
                sort: false,
            }
        },
        {
            name: "first_name",
            label: "First Name",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "last_name",
            label: "Last Name",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "email",
            label: "Email",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "channel",
            label: "Channel",
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: "position",
            label: "Position",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "position_abbr",
            label: "Position Abbr",
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: "team",
            label: "Team",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "tier",
            label: "Tier",
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: "site",
            label: "Site",
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: "team_manager",
            label: "Team Manager",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "active_queues",
            label: "Active Queues",
            options: {
                filter: false,
                sort: false,
                customBodyRender: (selectedOptions, {rowData}) => {
                    const {classes} = this.props;
                    return <TableColumnSelect
                        classes={classes}
                        selectedOptions={selectedOptions}
                        agentId={rowData[0]}
                        agentEmail={rowData[3]}
                        cacheName={'updatedAgentsTableQueues'}
                        shiftId={rowData[14]}
                        shiftStartTimeUTC={rowData[16]}
                        shiftEndTimeUTC={rowData[17]}
                    />
                }
            }
        },
        {
            name: "assigned_queues",
            label: "Assigned Queues",
            options: {
                filter: false,
                sort: false,
                customBodyRender: x => x.join(', '),
            }
        },
        {
            name: "off_reason",
            label: "Off Reason",
            options: {
                excluded: true,
                filter: false,
                sort: false,
            }
        },
        {
            name: "shift_id",
            label: "Shift ID",
            options: {
                filter: false,
                sort: false,
            }
        },
        {
            name: "shift_name",
            label: "Shift Name",
            options: {
                filter: false,
                sort: false,
            }
        },
        {
            name: "shift_start_time_utc",
            label: "Shift Start Time",
            options: {
                filter: false,
                sort: true,
                customBodyRender: x => this.dateToLocalString(x),
            }
        },
        {
            name: "shift_end_time_utc",
            label: "Shift End Time",
            options: {
                filter: false,
                sort: true,
                customBodyRender: x => this.dateToLocalString(x),
            }
        },
    ];

    dateToLocalString(rawDate) {
        return moment.utc(rawDate).local().format("D.M.YYYY. HH:mm")
    }

    parseAgents(response) {
        const cachedAgents = response.updatedAgentsTableQueues.agents;
        const cachedAgentIds = cachedAgents.map(x => x.agentId);

        return response.agents.map(x => {
            let activeQueues = [];
            const agentIndex = cachedAgentIds.indexOf(x.agent.id);
            if (agentIndex !== -1) {
                activeQueues = cachedAgents[agentIndex].activeQueues;
            } else {
                activeQueues = x.agent.active_queues.map(y => {
                    return {value: y.queue.id, label: y.queue.name.trim()};
                });
            }

            return [
                x.agent.id,
                x.agent.first_name,
                x.agent.last_name,
                x.agent.email,
                x.agent.channel,
                x.agent.position,
                x.agent.position_abbr,
                x.agent.team,
                x.agent.tier,
                x.agent.site,
                x.agent.team_manager ? x.agent.team_manager.email : '',
                updateToStringMethod(activeQueues),
                updateToStringMethod(
                    x.agent.assigned_queues.map(y => y.queue.name.trim())
                ),
                'off_reason' in x ? x.off_reason : null,
                x.shift.id,
                x.shift.name,
                x.start_time_utc,
                x.end_time_utc,
            ];
        });
    };

    render() {
        return (
            <Query query={this.getAgentControlsQuery}>
                {({loading, error, data}) => {
                    if (loading) return <Progress/>;
                    if (error) return <Error msg={'Failed to get local settings'} />;

                    const variables = parseAgentControlsQuery(data);
                    const {agentsTableColumnsOptions} = data;

                    const {isOnShift} = data;
                    const isDisplayed = {
                        agent_id: 'excluded',
                        shift_id: 'excluded',
                        off_reason: isOnShift ? 'excluded' : true,
                    };

                    const columns = updateColumnOptions(this.columns, agentsTableColumnsOptions, isDisplayed);

                    const tableName = isOnShift ? 'agent_on_shift' : 'agent_off_shift';
                    const title = `CS Agents ${isOnShift ? 'on' : 'off'} shift`;

                    const startDate = moment(data.selectedStartDate).format('YYYY-MM-DD-HH-mm');
                    const endDate = moment(data.selectedEndDate).format('YYYY-MM-DD-HH-mm');
                    const fileName = `agents-${isOnShift ? 'on' : 'off'}-shift_${startDate}_${endDate}.csv`;

                    return (
                        <Query query={this.getAgentsQuery(tableName)} variables={variables} fetchPolicy="cache-and-network">
                            {({loading, error, data, refetch}) => {
                                if (loading) return <Progress/>;
                                if (error) return <Error msg={'Failed to load agents'} />;

                                return (
                                    <AgentDataTable
                                        title={title}
                                        data={this.parseAgents(data)}
                                        columns={columns}
                                        customToolbar={() => <AgentsTableToolbar onApplyChanges={refetch}/>}
                                        downloadOptions={{filename: fileName}}
                                        persistDisplayedColumns={true}
                                        columnOptionsCacheName={'agentsTableColumnsOptions'}
                                        displayBulkQueuesEditToolbar={true}
                                        onApplyChanges={refetch}
                                    />
                                )
                            }}
                        </Query>
                    );
                }}
            </Query>
        )
    }
}

export const parseAgentControlsQuery = (data) => {
    return {
        start: moment(data.selectedStartDate).utc().format(),
        end: moment(data.selectedEndDate).utc().format(),
        channel: data.selectedChannels.length ? data.selectedChannels : null,
        team: data.selectedTeams.length ? data.selectedTeams : null,
        tier: data.selectedTiers.length ? data.selectedTiers : null,
        site: data.selectedSites.length ? data.selectedSites : null,
    };
};

export default withApollo(AgentsTable);
