import React from 'react';
import {Query, withApollo} from 'react-apollo';
import gql from 'graphql-tag';

import Tooltip from '@material-ui/core/Tooltip';
import Button from '@material-ui/core/Button';
import {Dialog, DialogActions, DialogContent, DialogTitle} from '@material-ui/core';
import {Step, StepLabel, Stepper} from '@material-ui/core';
import PropTypes from 'prop-types';
import moment from 'moment';

import {withStyles} from '@material-ui/core/styles';

import {cloneDeep} from 'lodash';
import Select from 'react-select';
import selectComponents, {selectStyles} from './autocompleteSelect';
import Progress from './progress';
import Error from './error';
import {QueueChangesTable, QueueChangesAction} from './tableTools';


const styles = theme => ({
    customToolbarSelect: {
        marginTop: theme.spacing.unit,
    },
    customToolbarItem: {
        marginRight: theme.spacing.unit * 2,
    },
    queueSelectWrapper: {
        minWidth: 552,
        minHeight: 400,
        padding: theme.spacing.unit * 2,
    },
});


class BulkQueuesEditToolbar extends React.Component {
    state = {
        dialog: {
            title: '',
            action: '',
            isOpen: false,
        },
    };

    handleAdd = () => {
        this.setState({
            dialog: {
                action: 'add',
                title: 'Add agents to queues',
                isOpen: true,
            }
        });
    };

    handleMove = () => {
        this.setState({
            dialog: {
                action: 'move',
                title: 'Move agents to queues',
                isOpen: true,
            }
        });
    };

    handleRemove = () => {
        this.setState({
            dialog: {
                action: 'remove',
                title: 'Remove agents from queues',
                isOpen: true,
            }
        });
    };

    closeDialog = () => {
        this.setState({dialog: {
            action: '',
            title: '',
            isOpen: false,
        }});
    };

    toUTCTimestamp = ts => {
        return moment(ts, 'D.M.YYYY. HH:mm').utc().format('YYYY-MM-DD HH:mm:ss')
    };

    getTableDataIndices() {
        const {columns} = this.props;
        let indices = {
            agent_id: -1,
            shift_id: -1,
            shift_start_time_utc: -1,
            shift_end_time_utc: -1,
            email: -1,
            active_queues: -1,
        };

        for (let i = 0; i < columns.length; i++) {
            if (columns[i].name in indices) {
                indices[columns[i].name] = i;
            }
        }
        return indices;
    }

    render() {
        const {classes, client, selectedRows, displayData, setSelectedRows} = this.props;
        const {dialog} = this.state;
        const indices = this.getTableDataIndices();

        const selectedAgents = selectedRows.data.map(row => {
            const queues = displayData[row.dataIndex].data[indices.active_queues].props.selectedOptions;
            return {
                agentId: displayData[row.dataIndex].data[indices.agent_id],
                shiftId: displayData[row.dataIndex].data[indices.shift_id],
                shiftStartTimeUTC: this.toUTCTimestamp(displayData[row.dataIndex].data[indices.shift_start_time_utc]),
                shiftEndTimeUTC: this.toUTCTimestamp(displayData[row.dataIndex].data[indices.shift_end_time_utc]),
                agentEmail: displayData[row.dataIndex].data[indices.email],
                activeQueues: queues,
            };
        });

        return (
            <React.Fragment>
                <BulkEditDialog isOpen={dialog.isOpen}
                                action={dialog.action}
                                title={dialog.title}
                                selectedAgents={selectedAgents}
                                setSelectedRows={setSelectedRows}
                                classes={classes}
                                client={client}
                                onClose={this.closeDialog}
                                onApplyChanges={this.props.onApplyChanges}
                />
                <div className={classes.customToolbarSelect}>
                    <Tooltip title="Add selected agents to queues"
                             className={classes.customToolbarItem}
                             enterDelay={500}
                    >
                        <Button variant="outlined" color="primary" onClick={this.handleAdd}>
                            Add to queues
                        </Button>
                    </Tooltip>
                    <Tooltip
                        title="Remove selected agents from the active queues and add them to new queues"
                        className={classes.customToolbarItem}
                        enterDelay={500}
                    >
                        <Button variant="outlined" color="primary" onClick={this.handleMove}>
                            Move to queues
                        </Button>
                    </Tooltip>
                    <Tooltip title="Remove selected agents from the queues"
                             className={classes.customToolbarItem}
                             enterDelay={500}
                    >
                        <Button variant="outlined" color="primary" onClick={this.handleRemove}>
                            Remove from queues
                        </Button>
                    </Tooltip>
                </div>
            </React.Fragment>
        );
    }
}

BulkQueuesEditToolbar.propTypes = {
    selectedRows: PropTypes.object.isRequired,
    displayData: PropTypes.array.isRequired,
    setSelectedRows: PropTypes.func.isRequired,
    columns: PropTypes.array.isRequired,
    onApplyChanges: PropTypes.func,
};

export default withApollo(withStyles((theme) => ({
      ...selectStyles(theme),
      ...styles(theme),
    }),
    { withTheme: true })(BulkQueuesEditToolbar));


class BulkEditDialog extends React.Component {
    steps = ['Choose queues', 'Confirm'];
    queuesQuery = gql`
        query {
           queue(order_by: {name:asc}) {
             id
             name
           }
        }
    `;

    constructor(props, context) {
        super(props, context);
        this.state = this.getDefaultState();
    }


    getDefaultState = () => {
      return {
          isOpen: false,
          action: false,
          title: '',
          confirmed: false,
          activeStep: 0,
          isNextEnabled: false,
          selectedQueues: [],
      };
    };

    setDialogOpen = () => {
        this.setState({isOpen: true});
    };

    setDialogClose = () => {
        const {onClose} = this.props;
        this.setState(this.getDefaultState());
        this.updateSelectedQueues([]);
        if (typeof onClose === 'function') {
            onClose();
        }
    };

    handleBack = () => {
        this.setState(state => ({activeStep: state.activeStep - 1}));
    };

    handleNext = () => {
        const {activeStep} = this.state;
        const {setSelectedRows} = this.props;

        if (activeStep >= this.steps.length - 1) {
            setSelectedRows([]);
            this.setDialogClose();
        } else {
            this.setState(state => ({activeStep: state.activeStep + 1}));
        }
    };

    componentWillReceiveProps(nextProps, nextContext) {
        const {isOpen, title} = nextProps;

        this.setState({title: title});
        if (isOpen) {
            this.setDialogOpen();
        }
    }

    renderQueueSelect() {
        const {classes} = this.props;
        return (
            <Query query={this.queuesQuery}>
                {({data, loading, error}) => {
                    if (loading) return <Progress/>;
                    if (error) return <Error msg={'Failed to load queues'}/>;

                    const options = data.queue.map(x => {
                        return {value: x.id, label: x.name};
                    });
                    return (
                        <div className={classes.queueSelectWrapper}>
                            <Select
                                isMulti
                                isClearable={false}
                                placeholder={'Select queues...'}
                                options={options}
                                defaultValue={this.state.selectedQueues}
                                classes={this.props.classes}
                                components={selectComponents}
                                noOptionsMessage={() => 'No queues'}
                                onChange={this.updateSelectedQueues}
                            />
                        </div>
                    );
                }}
            </Query>
        );
    }

    renderConfirmation() {
        return (
            <React.Fragment>
                <QueueChangesTable agents={this.parseUpdateAgentQueues()}/>
            </React.Fragment>
        );
    }

    parseUpdateAgentQueues() {
        const {selectedAgents, action} = this.props;
        const {selectedQueues} = this.state;

        const selectedQueueIds = selectedQueues.map(x => x.value);
        let agents = cloneDeep(selectedAgents);
        for (let i = 0; i < agents.length; i++) {
            if (action === 'add' || action === 'remove') {
                for (let j = 0; j < selectedQueueIds.length; j++) {
                    const activeQueueIds = agents[i].activeQueues.map(x => x.value);
                    const queueIndex = activeQueueIds.indexOf(selectedQueueIds[j]);
                    if (action === 'add' && queueIndex === -1) {
                        agents[i].activeQueues.push(selectedQueues[j]);
                    } else if (action === 'remove' && queueIndex !== -1) {
                        agents[i].activeQueues.splice(queueIndex, 1);
                    }
                }
            } else { // action === 'move'
                agents[i].activeQueues = selectedQueues;
            }
        }
        return agents;
    }

    updateSelectedQueues = updatedOptions => {
        this.setState({
            isNextEnabled: Boolean(updatedOptions.length),
            selectedQueues: [...updatedOptions],
        });
    };

    render() {
        const {activeStep} = this.state;
        return (
            <Dialog
                disableBackdropClick
                disableEscapeKeyDown
                open={this.state.isOpen}
                scroll="paper"
                maxWidth="lg"
            >
                <DialogTitle>{this.state.title}</DialogTitle>
                <DialogContent>
                    <Stepper activeStep={activeStep}>
                        {this.steps.map((label, index) => {
                            return (
                                <Step key={index}>
                                    <StepLabel>{label}</StepLabel>
                                </Step>
                            );
                        })}
                    </Stepper>
                    {activeStep === 0 ? this.renderQueueSelect() : this.renderConfirmation()}
                </DialogContent>
                <DialogActions>
                    <Button color="secondary" onClick={this.setDialogClose}>
                        Cancel
                    </Button>
                    <Button disabled={activeStep === 0} onClick={this.handleBack}>
                        Back
                    </Button>
                    {activeStep === this.steps.length - 1 ? (
                        <QueueChangesAction
                            agents={this.parseUpdateAgentQueues()}
                            onCompleted={() => {
                                const {onApplyChanges} = this.props;
                                if (typeof onApplyChanges === 'function') {
                                    onApplyChanges();
                                }
                                this.setDialogClose();
                            }}
                        />
                    ) : (
                        <Button
                            color="primary"
                            onClick={this.handleNext}
                            disabled={!this.state.isNextEnabled}
                        >
                            Next
                        </Button>
                    )}
                </DialogActions>
            </Dialog>
        )
    }
}

BulkEditDialog.propTypes = {
    title: PropTypes.string.isRequired,
    action: PropTypes.string.isRequired,
    client: PropTypes.object.isRequired,
    selectedAgents: PropTypes.array.isRequired,
    setSelectedRows: PropTypes.func.isRequired,
    onClose: PropTypes.func,
    onApplyChanges: PropTypes.func,
    isOpen: PropTypes.bool,
    classes: PropTypes.object,
};