import React, {Component} from 'react';
import PropTypes from 'prop-types';
import gql from 'graphql-tag';
import {Query, withApollo} from 'react-apollo';
import moment from 'moment';

import {List, ListItem, Fab, Typography} from '@material-ui/core';
import {withStyles} from '@material-ui/core/styles';

import ArrowBack from '@material-ui/icons/ArrowBack';

import grey from '@material-ui/core/colors/grey';
import teal from '@material-ui/core/colors/teal';

import QueuesTable from './queuesTable';
import QueuesControls from './queuesControls';
import Progress from './progress';
import Error from './error';
import Paper from "@material-ui/core/Paper";
import Grid from "@material-ui/core/Grid";


class QueuesDashboard extends Component {
    queryGroupsQuery = gql`
      query($now: timestamp!) {
        queue(order_by: {group_name: asc, name: asc}) {
          id
          name
          group_name
          agents: agent_shift_queues_aggregate(where: {
            agent: {
              agent_on_shifts: {
                start_time_utc: {_lte: $now},
                end_time_utc: {_gte: $now}
              }
            }
          }) {
            aggregate {
              count
            }
          }
        }
      }
    `;

    parseQueuesGroups(data) {
        let currentGroup = data.queue[0].group_name;
        let queueGroups = [{
            id: 1,
            name: currentGroup,
            inFocus: false,
            queues: []
        }];

        let groupIndex = 0;
        for (let i = 0; i < data.queue.length; i++) {
            const queue = data.queue[i];
            if (queue.group_name !== currentGroup) {
                groupIndex++;
                currentGroup = queue.group_name;

                queueGroups.push({
                    id: groupIndex + 1,
                    name: queue.group_name,
                    inFocus: false,
                    queues: []
                });
            }

            queueGroups[groupIndex].queues.push({
                id: queue.id,
                name: queue.name,
                agentCount: queue.agents.aggregate.count,
            });
        }
        return queueGroups;
    }

    render() {
        const {classes, client} = this.props;
        const now = moment.utc().format('YYYY-MM-DDTHH:mm');
        return (
            <Query query={this.queryGroupsQuery} variables={{now}} pollInterval={2500}>
                {({loading, error, data}) => {
                    if (loading) return <Progress/>;
                    if (error) return <Error msg={'Failed to get queue groups'}/>;
                    const queueGroups = this.parseQueuesGroups(data);
                    return <QueuesDashboardBody client={client} queueGroups={queueGroups} classes={classes}/>
                }}
            </Query>
        );
    }
}

QueuesDashboard.propTypes = {
    classes: PropTypes.object.isRequired,
    client: PropTypes.object.isRequired,
};

class QueuesDashboardBody extends Component {

    constructor(props, context) {
        super(props, context);
        this.state = {
            isFocusedView: false,
            queueGroups: this.props.queueGroups,
        };
    }

    componentWillReceiveProps(nextProps, nextContext) {
        const id2QueueGroups = this.state.queueGroups.reduce((acc, current) => {
            acc[current.id] = current.inFocus;
            return acc;
        }, {});

        let updatedQueueGroups = [...nextProps.queueGroups];

        for (let i = 0; i < updatedQueueGroups.length; i++) {
            if (updatedQueueGroups[i].id in id2QueueGroups) {
                updatedQueueGroups[i].inFocus = id2QueueGroups[updatedQueueGroups[i].id];
            } else {
                updatedQueueGroups[i].inFocus = false;
            }
        }

        this.setState({
            queueGroups: updatedQueueGroups,
        });
    }

    renderQueueGroups(groups) {
        const {classes} = this.props;
        const {isFocusedView} = this.state;
        let i = 0;
        return groups.map(g => {
            const width = isFocusedView && g.inFocus ? '100%' : '16.666666667%';
            if ((isFocusedView && g.inFocus) || !isFocusedView) {
                return (
                    <div
                        className={classes.groupWrapper}
                        style={{width: width}}
                        key={i++}
                    >
                        <QueuesGroup
                            id={g.id}
                            name={g.name || 'Unnamed'}
                            queues={g.queues}
                            classes={classes}
                            handleGroupClick={this.handleGroupClick}
                        />
                    </div>
                );
            } else {
                return null;
            }
        });
    }

    handleGroupClick = (groupId, queueId, queueName) => {
        let {queueGroups} = this.state;
        let {client} = this.props;
        for (let i = 0; i < queueGroups.length; i++) {
            queueGroups[i].inFocus = queueGroups[i].id === groupId;
        }

        const setQueuesQuery = gql`query { selectedQueues @client }`;
        const data = {
            selectedQueues: [{
                id: queueId,
                label: queueName,
                __typename: 'selectedQueues',
            }]
        };
        client.writeQuery({query: setQueuesQuery, data: data});

        this.setState({isFocusedView: true, queueGroups: queueGroups});
    };

    handleBackClick = () => {
        let {queueGroups} = this.state;
        for (let i = 0; i < queueGroups.length; i++) {
            queueGroups[i].inFocus = false;
        }

        this.setState({isFocusedView: false, queueGroups: queueGroups});
    };

    render() {
        const {queueGroups, isFocusedView} = this.state;
        const {classes} = this.props;
        return (
            <React.Fragment>
                <div className={classes.dashboardNavigation}>
                    {isFocusedView &&
                    <QueuesDashboardControls handleBackClick={this.handleBackClick}/>}
                </div>
                <div className={isFocusedView ? classes.dashboardFocused : classes.dashboard}>
                    {this.renderQueueGroups(queueGroups)}
                </div>

                {isFocusedView && (
                    <div className={classes.tableWrapper}>
                        <Paper><QueuesTable/></Paper>
                    </div>
                )}
            </React.Fragment>
        );
    }
}

QueuesDashboardBody.propTypes = {
    classes: PropTypes.object.isRequired,
    queueGroups: PropTypes.array.isRequired,
    client: PropTypes.object.isRequired,
};


class QueuesDashboardControls extends Component {

    render() {
        const {handleBackClick} = this.props;
        return (
            <React.Fragment>
                <Grid container spacing={16} justify="flex-start" alignItems="center">
                    <Grid item xs={2} style={{display: 'flex', justifyContent: 'center'}}>
                        <Fab color="primary" onClick={handleBackClick}>
                            <ArrowBack/>
                        </Fab>
                    </Grid>
                    <Grid item xs={10}>
                        <QueuesControls/>
                    </Grid>
                </Grid>
            </React.Fragment>
        );
    }
}

QueuesDashboardControls.propTypes = {
    handleBackClick: PropTypes.func.isRequired,
};


class QueuesGroup extends Component {

    renderItems(queues) {
        let i = 0;
        const {id, classes, handleGroupClick} = this.props;
        return queues.map(q => {
            return (
                <ListItem button key={i++}
                          className={classes.item}
                          onClick={() => {
                              handleGroupClick(id, q.id, q.name)
                          }}
                >
                    <span className={[classes.itemTitle, 'text'].join(' ')}>
                        {q.name}
                    </span>
                    <span className={[classes.itemDescription, 'text'].join(' ')}>
                        {q.agentCount} agents
                    </span>
                </ListItem>
            );
        });
    }

    render() {
        const {name, queues, classes} = this.props;
        return (
            <Paper className={[classes.controlWrapper, classes.groupPaper].join(' ')}>
                <div className={classes.groupHead}>
                    <Typography>{name}</Typography>
                </div>
                <List className={classes.group}>
                    {this.renderItems(queues)}
                </List>
            </Paper>
        );
    }
}

QueuesGroup.propTypes = {
    id: PropTypes.number.isRequired,
    name: PropTypes.string.isRequired,
    queues: PropTypes.array.isRequired,
    classes: PropTypes.object.isRequired,
    handleGroupClick: PropTypes.func.isRequired,
};

const styles = theme => ({
    group: {
        border: '1px solid',
        borderColor: theme.palette.type === 'light' ? grey[300] : grey[900],
        borderBottom: 0,
        borderBottomLeftRadius: 4,
        borderBottomRightRadius: 4,
        paddingTop: 0,
        paddingBottom: 0,
        overflowY: 'auto',
        maxHeight: 1500,
    },
    groupPaper: {
        margin: theme.spacing.unit,
    },
    groupWrapper: {
        display: 'inline-block',
    },
    item: {
        backgroundColor: theme.palette.type === 'light' ? grey[50] : grey[800],
        borderBottom: '1px solid',
        borderBottomColor: theme.palette.type === 'light' ? grey[300] : grey[900],
        justifyContent: 'left',
        flexDirection: 'column',
        alignItems: 'flex-start',
        '&:hover .text': {
            color: [teal[400], '!important'],
        },
    },
    itemTitle: {
        paddingTop: theme.spacing.unit,
        color: theme.palette.type === 'light' ? grey[900] : grey[200],
    },
    itemDescription: {
        fontSize: 14,
        color: theme.palette.type === 'light' ? grey[600] : grey[400],
    },
    groupHead: {
        display: 'flex',
        backgroundColor: theme.palette.type === 'light' ? grey[200] : grey[900],
        borderBottom: '1px solid',
        borderBottomColor: theme.palette.type === 'light' ? grey[300] : grey[900],
        borderTopLeftRadius: 4,
        borderTopRightRadius: 4,
        padding: theme.spacing.unit * 1.5,
        paddingLeft: theme.spacing.unit * 2,
        paddingRight: theme.spacing.unit * 2,
        '& p': {
            fontWeight: 500,
            color: theme.palette.type === 'light' ? grey[900] : theme.palette.primary.contrastText,
            opacity: theme.palette.type === 'light' ? 1 : 0.8,
        },
    },
    dashboard: {
        display: 'flex',
        flexDirection: 'column',
        flexWrap: 'wrap',
        maxHeight: 2600,
        paddingTop: theme.spacing.unit * 2,
        paddingBottom: theme.spacing.unit * 2,
        paddingLeft: theme.spacing.unit,
        paddingRight: theme.spacing.unit,
    },
    dashboardFocused: {
        display: 'inline-block',
        width: '16.67%',
        float: 'left',
        paddingTop: theme.spacing.unit * 2,
        paddingBottom: theme.spacing.unit * 2,
        paddingLeft: theme.spacing.unit,
        paddingRight: theme.spacing.unit,
    },
    dashboardNavigation: {
        minHeight: 88,
        padding: theme.spacing.unit * 2,
    },
    tableWrapper: {
        width: '83.33%',
        display: 'inline-block',
        paddingTop: theme.spacing.unit * 3,
        padding: theme.spacing.unit * 2,
    },
});

export default withApollo(withStyles(styles)(QueuesDashboard));
