import React, {Component} from 'react';

import {Query} from 'react-apollo';
import gql from 'graphql-tag';

import AgentDataTable from './table';
import {updateColumnOptions, updateToStringMethod} from './table';
import Progress from './progress';
import Error from './error';
import moment from 'moment';
import TableColumnSelect from './tableColumnSelect';
import QueuesTableToolbar from './queuesTableToolbar';


class QueuesTable extends Component {
    queuesQuery = gql`
        query($queueIds: [bigint!], $now: timestamp!) {
          updatedQueuesTableQueues @client {
            agents {
              agentId
              activeQueues @client
            }
          }
          queue(where: {id:{_in: $queueIds}}) {
            name
          }
          agents: agent(where: {
            agent_shift_queues: {
              queue_id: {_in: $queueIds}
            }
            agent_on_shifts: {
              start_time_utc: {_lte: $now},
              end_time_utc: {_gte: $now}
            }
          }) {
            id
            channel
            first_name
            last_name
            position
            position_abbr
            team
            tier
            site
            email
            agent_on_shifts(where: {
              start_time_utc: {_lte: $now},
              end_time_utc: {_gte: $now}
            }) {
              start_time_utc
              end_time_utc
              shift {
                id
                name
              }
            }
            team_manager: agent_team_manager {
              id
              email
            }
            active_queues: agent_shift_queues {
              queue {
                id
                name
              }
            }
            assigned_queues: agent_assigned_queues {
              queue {
                id
                name
              }
            }
          }
        }
    `;

    selectedQueuesQuery = gql`
      query {
        selectedQueues @client
        queuesTableColumnsOptions @client {
          name
          display
        }
      }`;

    columns = [
        {
            name: "agent_id",
            label: "Agent ID",
            options: {
                filter: false,
                sort: false,
            }
        },
        {
            name: "first_name",
            label: "First Name",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "last_name",
            label: "Last Name",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "email",
            label: "Email",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "channel",
            label: "Channel",
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: "position",
            label: "Position",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "position_abbr",
            label: "Position Abbr",
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: "team",
            label: "Team",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "tier",
            label: "Tier",
            options: {
                filter: true,
                sort: false,
            }
        },
        {
            name: "site",
            label: "Site",
            options: {
                filter: true,
                sort: false,
            }
        },
        {
            name: "team_manager",
            label: "Team Manager",
            options: {
                display: false,
                filter: false,
                sort: true,
            }
        },
        {
            name: "active_queues",
            label: "Active queues",
            options: {
                filter: false,
                sort: false,
                customBodyRender: (selectedOptions, {rowData}) => {
                    const {classes} = this.props;
                    return <TableColumnSelect
                        classes={classes}
                        cacheName={'updatedQueuesTableQueues'}
                        selectedOptions={selectedOptions}
                        agentId={rowData[0]}
                        agentEmail={rowData[3]}
                        shiftId={rowData[12]}
                        shiftStartTimeUTC={rowData[14]}
                        shiftEndTimeUTC={rowData[15]}
                    />
                }
            }
        },
        {
            name: 'shift_id',
            label: 'Shift ID',
            options: {
                filter: false,
                sort: false,
            }
        },
        {
            name: 'shift_name',
            label: 'Shift',
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: 'shift_start_time_utc',
            label: 'Shift Start Time',
            options: {
                filter: false,
                sort: true,
                customBodyRender: x => moment.utc(x).local().format('D.M.YYYY. HH:mm'),
            }
        },
        {
            name: 'shift_end_time_utc',
            label: 'Shift End Time',
            options: {
                filter: false,
                sort: true,
                customBodyRender: x => moment.utc(x).local().format('D.M.YYYY. HH:mm'),
            }
        },
        {
            name: "assigned_queues",
            label: "Assigned Queues",
            options: {
                filter: false,
                sort: false,
                customBodyRender: x => x.join(', '),
            }
        },
    ];

    parseAgents(response) {
        const cachedAgentQueues = response.updatedQueuesTableQueues.agents;
        const cachedAgentIds = cachedAgentQueues.map(x => x.agentId);

        return response.agents.map(x => {
            let activeQueues = [];

            const agentIndex = cachedAgentIds.indexOf(x.id);
            if (agentIndex !== -1) {
                activeQueues = cachedAgentQueues[agentIndex].activeQueues;
            } else {
                activeQueues = x.active_queues.map(y => (
                    {value: y.queue.id, label: y.queue.name.trim()}
                ));
            }

            let shift = {};
            try {
                shift = {
                    id: x.agent_on_shifts[0].shift.id,
                    name: x.agent_on_shifts[0].shift.name,
                    start: x.agent_on_shifts[0].start_time_utc,
                    end: x.agent_on_shifts[0].end_time_utc,
                }
            } catch (e) {
                shift = {id: -1, name: '', start: '', end: ''};
            }

            return [
                x.id,
                x.first_name,
                x.last_name,
                x.email,
                x.channel,
                x.position,
                x.position_abbr,
                x.team,
                x.tier,
                x.site,
                x.team_manager ? x.team_manager.email : '',
                updateToStringMethod(activeQueues),
                shift.id,
                shift.name,
                shift.start,
                shift.end,
                updateToStringMethod(
                    x.assigned_queues.map(y => y.queue.name.trim())
                ),
            ]
        });
    };

    getTitle(queueNames) {
        return queueNames.join(', ') + ' queue' + (queueNames.length > 1 ? 's' : '');
    }

    getFileName(queueNames) {
        const now = moment().local().format('DD-MM-YYYY-HH-mm');

        const reInvalidChars = /(\/|\\|:|\*|\?|\.|!|"|<|>|\|)+/gi;
        const reSpaceChars = /(\s|-|_)+/gi;
        let names = queueNames.map(
            q => q.replace(reInvalidChars, '').replace(reSpaceChars, '-')
        ).join('_');
        return `${names}_agents_${now}.csv`;
    }

    render() {
        return (
            <Query query={this.selectedQueuesQuery}>
                {({data, error, loading}) => {
                    if (loading) return null;
                    if (error) return <Error msg={'Failed to load queues data'} />;

                    const {selectedQueues, queuesTableColumnsOptions} = data;
                    const queueIds = selectedQueues.map(x => x.id);
                    const isDisplayed = {
                        agent_id: 'excluded',
                        shift_id: 'excluded',
                    };
                    const now = moment.utc().format('YYYY-MM-DDTHH:mm');
                    const columns = updateColumnOptions(
                        this.columns, queuesTableColumnsOptions, isDisplayed
                    );

                    return (
                        <Query query={this.queuesQuery} variables={{queueIds, now}} notifyOnNetworkStatusChange>
                            {({loading, error, data, refetch}) => {
                                if (loading) return <Progress/>;
                                if (error) return <Error msg={'Failed to load queue agents'} />;

                                const queueNames = data.queue.map(x => x.name);
                                return (
                                    <AgentDataTable
                                        title={this.getTitle(queueNames)}
                                        data={this.parseAgents(data)}
                                        columns={columns}
                                        customToolbar={() => <QueuesTableToolbar onApplyChanges={refetch}/>}
                                        displayBulkQueuesEditToolbar={true}
                                        downloadOptions={{
                                            filename: this.getFileName(queueNames)
                                        }}
                                        persistDisplayedColumns={true}
                                        columnOptionsCacheName={'queuesTableColumnsOptions'}
                                        onApplyChanges={refetch}
                                    />
                                )
                            }}
                        </Query>
                    )
                }}
            </Query>
        )
    }

}

export default QueuesTable;
