import React from 'react';
import gql from 'graphql-tag';
import {Query} from 'react-apollo';
import Select from 'react-select';
import AsyncSelect from 'react-select/lib/Async';
import PropTypes from 'prop-types';
import moment from 'moment';

import {QueueChangesTable, QueueChangesAction} from './tableTools';
import Error from './error';
import selectComponents from './autocompleteSelect';

import {
    Dialog,
    DialogActions,
    DialogContent,
    DialogTitle,
    Step,
    StepLabel,
    Stepper,
    Typography,
    Switch,
    FormControlLabel,
    Button,
} from '@material-ui/core';


class SingleAgentDialog extends React.Component {
    agentSelectInputValue = '';

    queuesQuery = gql`
        query {
           queue(order_by: {name:asc}) {
             id
             name
           }
        }
    `;

    selectedAgentsQuery = gql`
        query ($agents: [bigint!]!, $now: timestamp!) {
          agents: agent(where: {id: {_in: $agents}}) {
            id
            email
            shifts: agent_on_shifts(
              order_by: {start_time_utc: asc},
              where: {end_time_utc: {_gte: $now}},
              limit: 1
            ) {
              start_time_utc
              end_time_utc
              shift {
                id
                name
              }
            }
            queues: agent_shift_queues {
              queue {
                id
                name
              }
            }
          }
        }
    `;

    constructor(props, context) {
        super(props, context);
        this.state = this.getDefaultState();
        this.buildAgentsQuery();
    }

    buildAgentsQuery() {
        if (this.state.selectOnlyAgentsOnShift) {
            this.agentsQuery = gql`
                query ($email: String!, $now: timestamp!) {
                  agent (order_by: {email: asc}, where: {
                    email: {_ilike: $email},
                    agent_on_shifts: {start_time_utc: {_lte: $now}, end_time_utc: {_gte: $now}}
                  }) {
                    id
                    email
                  }
                }
            `;
        } else {
            this.agentsQuery = gql`
                query($email: String!) {
                  agent (order_by: {email: asc}, where: {
                    email: {_ilike: $email},
                  }) {
                    id
                    email
                  }
                }
            `;
        }
    }

    getDefaultState = () => {
      return {
          isOpen: false,
          title: '',
          confirmed: false,
          activeStep: 0,
          isNextEnabled: [false],
          selectedQueues: [],
          selectedAgents: [],
          steps: [],
          agentSelectInputValue: '%',
          selectOnlyAgentsOnShift: true,
          predefinedQueues: false,
      };
    };

    setDialogOpen = () => {
        this.setState({isOpen: true});
    };

    setDialogClose = () => {
        this.setState(this.getDefaultState());

        const {onClose} = this.props;
        if (typeof onClose === 'function') {
            onClose();
        }
    };

    handleBack = () => {
        this.setState(state => ({activeStep: state.activeStep - 1}));
    };

    handleNext = () => {
        const {activeStep, steps} = this.state;
        const {setSelectedRows} = this.props;

        if (activeStep >= steps.length - 1) {
            setSelectedRows([]);
            this.setDialogClose();
        } else {
            this.setState(state => ({activeStep: state.activeStep + 1}));
        }
    };


    componentWillReceiveProps(nextProps, nextContext) {
        const {isOpen, title, selectedQueues} = nextProps;
        this.setState(this.getDefaultState());

        if (selectedQueues.length) {
            this.setState({
                title: title,
                selectedQueues: selectedQueues,
                predefinedQueues: true,
                steps: ['Choose agents', 'Confirm'],
            });
        } else {
            this.setState({
                title: title,
                isNextEnabled: [false, false],
                steps: ['Choose agents', 'Choose queues', 'Confirm'],
            });
        }

        if (isOpen) {
            this.setDialogOpen();
        }
    }

    renderQueueSelect() {
        const {classes} = this.props;
        return (
            <Query query={this.queuesQuery}>
                {({data, loading, error}) => {
                    if (loading) return null;
                    if (error) return <Error msg={'Failed to load queues'}/>;

                    const options = data.queue.map(x => {
                        return {value: x.id, label: x.name};
                    });
                    return (
                        <div className={classes.selectWrapper}>
                            <Select
                                isMulti
                                isClearable={false}
                                placeholder={'Select queues...'}
                                options={options}
                                defaultValue={this.state.selectedQueues}
                                classes={this.props.classes}
                                components={selectComponents}
                                noOptionsMessage={() => 'No queues'}
                                onChange={this.updateSelectedQueues}
                            />
                        </div>
                    );
                }}
            </Query>
        );
    }

    loadAgentSelectOptions = (inputValue, callback) => {
        setTimeout(() => {
            if (this.lastAgentSelectInputValue === this.agentSelectInputValue) {
                return callback(this.lastAgentSelectOptions);
            }
            const {client} = this.props;
            const currentAgentSelectInputValue = this.agentSelectInputValue;

            let variables = {email:`%${currentAgentSelectInputValue}%`};
            if (this.state.selectOnlyAgentsOnShift) {
                variables.now = moment.utc().format('YYYY-MM-DDTHH:mm');
            }

            client.query({
                query: this.agentsQuery,
                variables: variables,
            }).then(response => {
                const options = response.data.agent.map(x => {
                    return {value: x.id, label: x.email};
                });

                this.lastAgentSelectInputValue = currentAgentSelectInputValue;
                this.lastAgentSelectOptions = options;
                callback(options);
            });
        }, 1500);
    };

    onAgentSelectInputChange = newValue => {
        this.agentSelectInputValue = newValue;
        return newValue;
    };

    toggleOnlyAgentsOnShift = event => {
        this.setState({
            selectOnlyAgentsOnShift: event.target.checked,
        }, this.buildAgentsQuery);
    };

    renderAgentSelect() {
        const {classes} = this.props;
        return (
            <React.Fragment>
                <div className={classes.selectWrapper}>
                    <AsyncSelect
                        isMulti
                        isClearable={false}
                        placeholder={'Select agents...'}
                        defaultValue={this.state.selectedAgents}
                        classes={this.props.classes}
                        components={selectComponents}
                        loadOptions={this.loadAgentSelectOptions}
                        cacheOptions
                        noOptionsMessage={() => 'No agents'}
                        onChange={this.updateSelectedAgents}
                        onInputChange={this.onAgentSelectInputChange}
                    />
                    <FormControlLabel
                        control={
                            <Switch
                                checked={this.state.selectOnlyAgentsOnShift}
                                onChange={this.toggleOnlyAgentsOnShift}
                                value="on-shift"
                                color="primary"
                            />
                        }
                        label="Only agents currently on shift"
                    />
                </div>

            </React.Fragment>
        )
    }

    renderConfirmation() {
        return this.fetchSelectedAgentsAdditionalData(agents => {
            return (
                <div>
                    <Typography>
                        Queues will be activated for the chosen agents and
                        their active or closest shift from now, if any.
                    </Typography>
                    <QueueChangesTable
                        agents={this.parseUpdateAgentQueues(agents, this.toDisplayDateTimeString)}
                        displayRemoved={false}
                        additionalAgentsKeys={['shiftName', 'shiftStartTimeUTC', 'shiftEndTimeUTC']}
                        additionalLabels={['Shift Name', 'Start Time', 'End Time']}
                    />
                </div>
            );
        });
    }

    fetchSelectedAgentsAdditionalData = callback => {
      const {selectedAgents} = this.state;
        const variables = {
            agents: selectedAgents.map(x => x.value),
            now: moment.utc().format('YYYY-MM-DDTHH:mm'),
        };
        return (
            <Query query={this.selectedAgentsQuery} variables={variables}>
                {({data, loading, error}) => {
                    if (loading) return null;
                    if (error) return <Error msg={'Failed to load selected agents'}/>;
                    return callback(data.agents);
                }}
            </Query>
        );
    };

    toTimestampString = ts => {
        return moment.utc(ts).local().format('YYYY-MM-DDTHH:mm');
    };

    toDisplayDateTimeString = ts => {
        return moment.utc(ts).local().format('D.M.YYYY. HH:mm');
    };

    parseUpdateAgentQueues(rawAgents, tsParseFunction) {
        const {selectedQueues} = this.state;
        let agents = rawAgents.map(a => {
            return {
                agentId: a.id,
                shiftId: a.shifts.length ? a.shifts[0].shift.id : null,
                shiftName: a.shifts.length ? a.shifts[0].shift.name : '',
                shiftStartTimeUTC: a.shifts.length ? tsParseFunction(a.shifts[0].start_time_utc) : null,
                shiftEndTimeUTC: a.shifts.length ? tsParseFunction(a.shifts[0].end_time_utc) : null,
                agentEmail: a.email,
                activeQueues: a.queues.map(q => {
                    return {value: q.queue.id, label: q.queue.name}
                }),
            };
        });

        const selectedQueueIds = selectedQueues.map(x => x.value);
        for (let i = 0; i < agents.length; i++) {
            for (let j = 0; j < selectedQueueIds.length; j++) {
                const activeQueueIds = agents[i].activeQueues.map(x => x.value);
                const queueIndex = activeQueueIds.indexOf(selectedQueueIds[j]);
                if (queueIndex === -1) {
                    agents[i].activeQueues.push(selectedQueues[j]);
                }
            }
        }
        return agents;
    }

    updateSelectedAgents = updatedOptions => {
        this.setState(state => {
            let isNextEnabled = state.isNextEnabled;
            isNextEnabled[0] = Boolean(updatedOptions.length);
            return {
                isNextEnabled: isNextEnabled,
                selectedAgents: [...updatedOptions],
            };
        });
    };

    updateSelectedQueues = updatedOptions => {
        this.setState(state => {
            let isNextEnabled = state.isNextEnabled;
            isNextEnabled[isNextEnabled.length - 1] = Boolean(updatedOptions.length);
            return {
                isNextEnabled: isNextEnabled,
                selectedQueues: [...updatedOptions],
            };
        });
    };

    render() {
        const {activeStep, title, isOpen, predefinedQueues, steps} = this.state;
        return (
            <Dialog
                disableBackdropClick
                disableEscapeKeyDown
                open={isOpen}
                scroll="paper"
                fullWidth={true}
                maxWidth="lg"
            >
                <DialogTitle>{title}</DialogTitle>

                <DialogContent>
                    <Stepper activeStep={activeStep}>
                        {steps.map((label, index) => {
                            return (
                                <Step key={index}>
                                    <StepLabel>{label}</StepLabel>
                                </Step>
                            );
                        })}
                    </Stepper>
                    {activeStep === 0 && this.renderAgentSelect()}
                    {(activeStep === 1 && !predefinedQueues) && this.renderQueueSelect()}
                    {((activeStep === 1 && predefinedQueues) || activeStep === 2) && this.renderConfirmation()}
                </DialogContent>

                <DialogActions>
                    <Button color="secondary" onClick={this.setDialogClose}>
                        Cancel
                    </Button>
                    <Button disabled={activeStep === 0} onClick={this.handleBack}>
                        Back
                    </Button>
                    {activeStep === steps.length - 1 ? (
                        this.fetchSelectedAgentsAdditionalData(agents => {
                            return (
                                <QueueChangesAction
                                    agents={this.parseUpdateAgentQueues(agents, this.toTimestampString)}
                                    onCompleted={() => {
                                        const {onApplyChanges} = this.props;
                                        if (typeof onApplyChanges === 'function') {
                                            onApplyChanges();
                                        }
                                        this.setDialogClose();
                                    }}
                                />
                            );
                        })
                    ) : (
                        <Button
                            color="primary"
                            onClick={this.handleNext}
                            disabled={!this.state.isNextEnabled[activeStep]}
                        >
                            Next
                        </Button>
                    )}
                </DialogActions>
            </Dialog>
        )
    }
}

SingleAgentDialog.propTypes = {
    title: PropTypes.string.isRequired,
    client: PropTypes.object.isRequired,
    selectedAgents: PropTypes.array,
    selectedQueues: PropTypes.array,
    setSelectedRows: PropTypes.func,
    onClose: PropTypes.func,
    onApplyChanges: PropTypes.func,
    isOpen: PropTypes.bool,
    classes: PropTypes.object,

};

SingleAgentDialog.defaultProps = {
    selectedAgents: [],
    selectedQueues: [],
};

export default SingleAgentDialog;