import React, {Component} from 'react';
import MUIDataTable from 'mui-datatables';
import PropTypes from 'prop-types';
import gql from 'graphql-tag';
import {withApollo} from 'react-apollo';

import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';

import BulkQueuesEditToolbar from './bulkQueuesEditToolbar';
import moment from 'moment';


class AgentDataTable extends Component {

    constructor(props, context) {
        super(props, context);
        this.table_options = {
            print: false,
            rowsPerPage: 20,
            rowsPerPageOptions: [10, 20, 50, 100],
            filterType: 'checkbox',
            textLabels: {
                selectedRows: {
                    text: 'agent(s) selected',
                },
                body: {
                    noMatch: 'Sorry, no matching agents found',
                },
            },
        };
        this.columns = [];
    }

    onColumnOptionsChange = (action, tableState) => {
        if (action !== 'columnViewChange') {
            return;
        }
        const isDisplayed = tableState.columns.reduce((displayed, column) => {
            displayed[column.name] = column.display === 'true' ? true : false;
            return displayed;
        }, {});

        const {client, columnOptionsCacheName} = this.props;
        const query = gql`
            query {
              ${columnOptionsCacheName} @client {
                name,
                display
              }
            }
        `;
        const data = client.readQuery({query: query});

        let updatedData = {...data};
        for (let i = 0; i < updatedData[columnOptionsCacheName].length; i++) {
            const name = updatedData[columnOptionsCacheName][i].name;
            if (name in isDisplayed) {
                updatedData[columnOptionsCacheName][i]['display'] =
                    isDisplayed[name] ? 'true' : 'false';
            }
        }
        client.writeQuery({query: query, data: updatedData});
    };

    render() {
        const {
            title,
            data,
            columns,
            onTableChange,
            customToolbar,
            downloadOptions,
            displayBulkQueuesEditToolbar,
            persistDisplayedColumns
        } = this.props;

        this.table_options['onTableChange'] = (action, tableState) => {
            if (persistDisplayedColumns) {
                this.onColumnOptionsChange(action, tableState);
            }
            if (typeof onTableChange === 'function') {
                onTableChange(action, tableState);
            }
        };

        if (typeof customToolbar === 'function') {
            this.table_options['customToolbar'] = customToolbar;
        }

        if (typeof downloadOptions === 'object') {
            this.table_options['downloadOptions'] = downloadOptions;
        }

        if (displayBulkQueuesEditToolbar) {
            const {onApplyChanges} = this.props;
            this.table_options['customToolbarSelect'] = (selectedRows, displayData, setSelectedRows) => (
                <BulkQueuesEditToolbar selectedRows={selectedRows}
                                       displayData={displayData}
                                       setSelectedRows={setSelectedRows}
                                       onApplyChanges={onApplyChanges}
                                       columns={columns}
                />
            );
        }

        return (
            <MUIDataTable
                title={title}
                data={data}
                columns={columns}
                options={this.table_options}
            />
        )
    }
}

AgentDataTable.propTypes = {
    classes: PropTypes.object,
    client: PropTypes.object.isRequired,
    title: PropTypes.string.isRequired,
    data: PropTypes.array.isRequired,
    columns: PropTypes.array.isRequired,
    onTableChange: PropTypes.func,
    onApplyChanges: PropTypes.func,
    columnOptionsCacheName: PropTypes.string,
    persistDisplayedColumns: PropTypes.bool,
    customToolbar: PropTypes.func,
    displayBulkQueuesEditToolbar: PropTypes.bool,
    downloadOptions: PropTypes.object,
};

AgentDataTable.defaultProps = {
    displayBulkQueuesEditToolbar: false,
    columnOptionsCacheName: false,
    persistDisplayedColumns: false,
};

export default withApollo(AgentDataTable);


export class HistoryTable extends Component {
    constructor(props, context) {
        super(props, context);

        const now = moment().local().format('YYYY-MM-DD-HH-mm');
        this.table_options = {
            print: false,
            selectableRows: false,
            rowsPerPage: 50,
            rowsPerPageOptions: [20, 50, 100, 200],
            filterType: 'select',
            downloadOptions: {filename: `history_${now}.csv`},
        };
    }

    columns = [
        {
            name: "date",
            label: "Date",
            options: {
                filter: false,
                sort: true,
                customBodyRender: x => moment.utc(x).local().format("D.M.YYYY. HH:mm"),
            }
        },
        {
            name: "action",
            label: "Action",
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: "agent",
            label: "Agent Email",
            options: {
                filter: false,
                sort: true,
            }
        },
        {
            name: "agentId",
            label: "Agent ID",
            options: {
                display: false,
                excluded: true,
                filter: false,
                sort: false,
            }
        },
        {
            name: "queue",
            label: "Queue",
            options: {
                filter: true,
                sort: true,
            }
        },
        {
            name: "queueId",
            label: "Queue ID",
            options: {
                display: false,
                excluded: true,
                filter: false,
                sort: false,
            }
        },
        {
            name: "author",
            label: "Author",
            options: {
                filter: false,
                sort: false,
            }
        },
    ];

    render() {
        const {data} = this.props;
        return (
            <MUIDataTable
                title="History"
                data={data}
                columns={this.columns}
                options={this.table_options}
            />
        )
    }
}

HistoryTable.propTypes = {
    classes: PropTypes.object,
    data: PropTypes.array.isRequired,
};


export class SimpleTable extends Component {

    constructor(props, context) {
        super(props, context);
        const {rows, labels} = this.props;

        this.state = {};

        let rowCounter = 0;
        this.state.rows = rows.map(columns => {
            let columnCounter = 0;
            columns = columns.map(x => {
                return {id: columnCounter++, value: x}
            });
            return {id: rowCounter++, columns: columns};
        });

        if (labels) {
            let labelCounter = 0;
            this.state.labels = labels.map(x => {
                return {id: labelCounter++, value: x};
            });
        } else {
            this.state.labels = [];
        }
    }


    render() {
        const {rows, labels} = this.state;
        return (
            <Table>
                {labels.length &&
                <TableHead>
                    <TableRow>
                        {labels.map(label => (
                            <TableCell key={label.id}>
                                {label.value}
                            </TableCell>
                        ))}
                    </TableRow>
                </TableHead>
                }
                <TableBody>
                    {rows.map(row => (
                        <TableRow key={row.id}>
                            {row.columns.map(column => (
                                <TableCell key={column.id}>
                                    {column.value}
                                </TableCell>
                            ))}
                        </TableRow>
                    ))}
                </TableBody>
            </Table>
        );
    }
}

SimpleTable.propTypes = {
    rows: PropTypes.array.isRequired,
    labels: PropTypes.array,
};


export function updateColumnOptions(columns, newOptions, isDisplayedOverride) {
    let isDisplayed = newOptions.reduce((displayed, column) => {
        displayed[column.name] = column.display;
        return displayed;
    }, {});

    if (typeof isDisplayedOverride === 'object') {
        isDisplayed = {...isDisplayed, ...isDisplayedOverride};
    }

    for (let i = 0; i < columns.length; i++) {
        if (isDisplayed[columns[i]['name']] !== undefined) {
            columns[i].options['display'] = isDisplayed[columns[i]['name']];
        }
    }
    return columns;
}


export function updateToStringMethod(queues) {
    /**
     * Replace queues array's toString method for usage in CSV download
     * toolbar option. Use "|" for delimiter.
     * @param queues {array of string or objects}
     * @returns {array}
     */
    queues.toString = function () {
        let output = '';
        for (let i = 0; i < this.length; i++) {
            if (output.length) {
                output += '|';
            }
            if (typeof this[i] === 'string') {
                output += this[i];
            } else if (typeof this[i] === 'object') {
                output += this[i].label;
            }
        }
        return output;
    };
    return queues;
}