import React, {Component} from 'react';
import {Query, withApollo} from 'react-apollo';
import gql from 'graphql-tag';

import {withStyles} from '@material-ui/core/styles';
import {ClickAwayListener, Tooltip} from '@material-ui/core';
import Select from 'react-select';
import selectComponents, {selectStyles} from './autocompleteSelect';
import PropTypes from 'prop-types';
import Progress from './progress';
import Error from './error';


function createStyled(styles, options) {
    function Styled(props) {
        const {children, ...other} = props;
        return children(other);
    }

    Styled.propTypes = {
        children: PropTypes.func.isRequired,
        classes: PropTypes.object.isRequired,
    };
    return withStyles(styles, options)(Styled);
}

const LocalStyle = createStyled({
    previewWrapper: {
        '&:hover': {
            cursor: 'pointer',
        }
    }
});


class TableColumnSelect extends Component {
    queuesQuery = gql`
        query {
           queue(order_by: {name:asc}) {
             id
             name
           }
        }
    `;

    constructor(props, context) {
        super(props, context);
        this.state = {
            isEdit: false,
        }
    }

    toggleIsEdit = () => {
        this.setState({isEdit: !this.state.isEdit});
    };

    updateCache = (updatedOptions) =>  {
        const {
            client, cacheName, agentId, agentEmail, shiftId, 
            shiftStartTimeUTC, shiftEndTimeUTC
        } = this.props;
        const id = `${agentId}-${shiftId}`;
        const query = gql`
            query {
              ${cacheName} @client {
                  agents {
                    id
                    agentId
                    agentEmail
                    ${shiftId !== undefined ? 'shiftId' : ''}
                    ${shiftStartTimeUTC !== undefined ? 'shiftStartTimeUTC' : ''}
                    ${shiftEndTimeUTC !== undefined ? 'shiftEndTimeUTC' : ''}
                    activeQueues @client
                  }
              }
            }
        `;

        const data = client.readQuery({query: query});
        const ids = data[cacheName].agents.map(x => x.id);
        const agentIndex = ids.indexOf(id);
        const agent = {
            id: id,
            agentId: agentId,
            agentEmail: agentEmail,
            shiftId: shiftId,
            shiftStartTimeUTC: shiftStartTimeUTC,
            shiftEndTimeUTC: shiftEndTimeUTC,
            activeQueues: updatedOptions,
            __typename: 'updatedAgentQueues',
        };

        let updatedData = {...data};
        if (agentIndex !== -1) {
            updatedData[cacheName].agents.splice(agentIndex, 1, agent);
        } else {
            updatedData[cacheName].agents.push(agent);
        }

        client.writeQuery({query: query, data: updatedData});
    };

    render() {
        const {classes} = this.props;
        const {isEdit} = this.state;
        const {selectedOptions} = this.props;
        return (
            <React.Fragment>
                {!isEdit && (
                    <LocalStyle>
                        {({classes}) => (
                            <Tooltip title={'Edit this agent\'s queues'} enterDelay={800}>
                                <div
                                    className={classes.previewWrapper}
                                    onDoubleClick={this.toggleIsEdit}
                                >
                                    {selectedOptions.length ?
                                        selectedOptions.map(x => x.label).join(', ') : 'None'}
                                </div>
                            </Tooltip>
                        )}
                    </LocalStyle>
                )}

                {isEdit && (
                    <Query query={this.queuesQuery}>
                        {({data, loading, error}) => {
                            if (loading) return <Progress/>;
                            if (error) return <Error msg={'Failed to load queues'} />;

                            const options = data.queue.map(x => {
                                return {value: x.id, label: x.name};
                            });
                            return (
                                <ClickAwayListener onClickAway={this.toggleIsEdit}>
                                    <Select
                                        isMulti
                                        isClearable={false}
                                        placeholder={''}
                                        options={options}
                                        classes={classes}
                                        components={selectComponents}
                                        defaultValue={selectedOptions}
                                        noOptionsMessage={() => 'No queues'}
                                        onChange={this.updateCache}
                                    />
                                </ClickAwayListener>
                            );
                        }}
                    </Query>
                )}
            </React.Fragment>
        );
    }
}

TableColumnSelect.propTypes = {
    classes: PropTypes.object.isRequired,
    selectedOptions: PropTypes.array.isRequired,
    agentId: PropTypes.number.isRequired,
    agentEmail: PropTypes.string.isRequired,
    cacheName: PropTypes.string.isRequired,
    shiftId: PropTypes.number,
    shiftStartTimeUTC: PropTypes.string,
    shiftEndTimeUTC: PropTypes.string,
};

export default withApollo(withStyles(selectStyles)(TableColumnSelect));
