import React from 'react';
import {
    Typography,
    Button, IconButton,
    Tooltip,
    Dialog, DialogTitle, DialogContent, DialogActions
} from '@material-ui/core';

import DomainIcon from '@material-ui/icons/Domain';
import DomainDisabledIcon from '@material-ui/icons/DomainDisabled';

import {Mutation, Query} from 'react-apollo';
import gql from 'graphql-tag';
import PropTypes from 'prop-types';
import {flatten} from 'lodash';
import moment from 'moment';

import Progress from './progress';
import Error from './error';
import {parseAgentControlsQuery} from './agentsTable';
import {SimpleTable} from './table';


export class ApplyChangesConfirmation extends React.Component {
    state = {
        discardDialogOpen: false,
        confirmationDialogOpen: false,
        confirmed: false,
    };

    constructor(props, context) {
        super(props, context);
        const {cacheName, additionalColumns} = this.props;

        let additionalColumnsNames = [];
        if (additionalColumns) {
            additionalColumnsNames = Object.values(additionalColumns);
        }
        this.cacheQuery = gql`
            query {
              ${cacheName} @client {
                  agents {
                    id
                    agentId
                    agentEmail
                    shiftId
                    shiftStartTimeUTC
                    shiftEndTimeUTC
                    activeQueues @client
                    ${additionalColumnsNames.join('\n')}
                  }
              }
            }
        `;
    }

    setConfirmationDialogOpen = () => {
        this.setState({confirmationDialogOpen: true});
    };

    setConfirmationDialogClose = () => {
        this.setState({confirmationDialogOpen: false});
    };

    setDiscardDialogOpen = () => {
        this.setState({discardDialogOpen: true});
    };

    setDiscardDialogClose = () => {
        this.setState({discardDialogOpen: false});
    };

    cleanCache = () => {
        const {client, cacheName, onApplyChanges} = this.props;
        let data = {};
        data[cacheName] = {
            __typename: cacheName,
            agents: [],
        };
        client.writeQuery({query: this.cacheQuery, data: data});

        if (typeof onApplyChanges === 'function') {
            onApplyChanges();
        }
    };

    renderToolbarButtons = () => {
        return (
            <React.Fragment>
                <Button
                    size="small"
                    variant="contained"
                    color="primary"
                    onClick={this.setConfirmationDialogOpen}
                    style={{marginLeft: 12}}
                >
                    Apply Changes
                </Button>
                <Button
                    size="small"
                    variant="contained"
                    color="secondary"
                    onClick={this.setDiscardDialogOpen}
                    style={{marginLeft: 12}}
                >
                    Discard Changes
                </Button>
            </React.Fragment>
        );
    };

    renderConfirmationDialog = agents => {
        const {additionalColumns, classes} = this.props;

        return (
            <Dialog
                disableBackdropClick
                disableEscapeKeyDown
                maxWidth="lg"
                open={this.state.confirmationDialogOpen}
            >
                <DialogTitle>Confirm Changes</DialogTitle>
                <DialogContent>
                    <React.Fragment>
                        <Typography>
                            Next agents will have their active queues updated:
                        </Typography>
                        <br/>
                        <QueueChangesTable agents={agents} additionalColumns={additionalColumns}/>
                    </React.Fragment>
                </DialogContent>
                <DialogActions className={classes.dialogActions}>
                    <Button color="secondary" onClick={this.setConfirmationDialogClose}>
                        Cancel
                    </Button>
                    <QueueChangesAction agents={agents} onCompleted={() => {
                        this.cleanCache();
                        this.setConfirmationDialogClose();
                    }}/>
                </DialogActions>
            </Dialog>
        );
    };

    renderDiscardDialog = agents => {
        const {classes} = this.props;
        return (
            <Dialog
                disableBackdropClick
                disableEscapeKeyDown
                maxWidth="lg"
                open={this.state.discardDialogOpen}
            >
                <DialogTitle>Discard Changes</DialogTitle>
                <DialogContent>
                    <Typography>
                        Are you sure you want to discard all unsaved changes?
                    </Typography>
                    <br/>
                    <QueueChangesTable agents={agents}/>
                </DialogContent>
                <DialogActions className={classes.dialogActions}>
                    <Button color="secondary" onClick={this.setDiscardDialogClose}>
                        No
                    </Button>
                    <Button color="primary" onClick={() => {
                        this.cleanCache();
                        this.setDiscardDialogClose();
                    }}
                    >
                        Yes
                    </Button>
                </DialogActions>
            </Dialog>
        );
    };

    render() {
        const {cacheName} = this.props;
        return (
            <Query query={this.cacheQuery}>
                {({data, loading, error}) => {
                    if (loading) return <Progress/>;
                    if (error) return <Error msg={'Failed to fetch agent\'s old queues'}/>;
                    const changesCount = data[cacheName].agents.length;

                    if (changesCount) {
                        const agents = data[cacheName].agents;
                        return (
                            <React.Fragment>
                                {this.renderToolbarButtons()}
                                {this.renderConfirmationDialog(agents)}
                                {this.renderDiscardDialog(agents)}
                            </React.Fragment>
                        );
                    } else {
                        return <React.Fragment></React.Fragment>
                    }
                }}
            </Query>
        )
    }
}

ApplyChangesConfirmation.propTypes = {
    classes: PropTypes.object.isRequired,
    client: PropTypes.object.isRequired,
    cacheName: PropTypes.string.isRequired,
    /**
     * Should be an object where key is name of the column in DB, and value is
     * name of the column in local cache.
     */
    additionalColumns: PropTypes.object,
    onApplyChanges: PropTypes.func,
};


export class QueueChangesTable extends React.Component {
    agentQueuesQuery = gql`
        query($agents: [bigint!]!) {
          agents: agent(where: {id: {_in: $agents}}) {
            id
            queues: agent_shift_queues {
              queue {
                id
                name
              }
            }
          }
        }
    `;

    getTableLabels = () => {
        const {displayRemoved, additionalLabels} = this.props;
        let labels = ['Agent'];

        if ( additionalLabels.length ) {
            labels = labels.concat( additionalLabels );
        }

        labels.push('Added');

        if ( displayRemoved ) {
            labels.push('Removed');
        }

        labels.push('New active queues');
        return labels;
    };

    getTableData = data => {
        const {agents, displayRemoved, additionalAgentsKeys} = this.props;
        const agentOldQueues = data.agents.reduce((acc, val) => {
            acc[val.id] = val.queues.map(x => x.queue.name);
            return acc;
        }, {});

        return agents.map(agent => {
            const oldQueues = agentOldQueues[agent.agentId];
            const newQueues = agent.activeQueues.map(q => q.label);

            let row = [agent.agentEmail];
            for (let i = 0; i < additionalAgentsKeys.length; i++) {

                row.push(agent[additionalAgentsKeys[i]] || 'None');
            }

            row.push(newQueues.filter(x => !oldQueues.includes(x)).join(', ') || 'None');
            if (displayRemoved) {
                row.push(oldQueues.filter(x => !newQueues.includes(x)).join(', ') || 'None');
            }

            row.push(newQueues.join(', ') || 'None');
            return row;
        });
    };

    render() {
        const {agents} = this.props;
        const agentIds = agents.map(x => x.agentId);
        return (
            <Query query={this.agentQueuesQuery} variables={{agents: agentIds}}>
                {({data, loading, error}) => {
                    if (loading) return <Progress/>;
                    if (error) return <Error msg={'Failed to fetch agent\'s old queues'}/>;

                    return (
                        <SimpleTable
                            rows={this.getTableData(data)}
                            labels={this.getTableLabels()}
                        />
                    );
                }}
            </Query>
        );
    }
}

QueueChangesTable.propTypes = {
    agents: PropTypes.array.isRequired,
    displayRemoved: PropTypes.bool,
    additionalAgentsKeys: PropTypes.array,
    additionalLabels: PropTypes.array,
};

QueueChangesTable.defaultProps = {
    displayRemoved: true,
    additionalAgentsKeys: [],
    additionalLabels: [],
};

const currentUserEmail = "test-bp@test-bp.com"

export class QueueChangesAction extends React.Component {

    flattenAgentQueues = (agents, now, isActive) => {
        const {additionalColumns} = this.props;

        return flatten(agents.map(a => {
            return a.activeQueues.map(q => {
                let agent = {
                    shift_id: a.shiftId,
                    start_time_utc: a.shiftStartTimeUTC,
                    end_time_utc: a.shiftEndTimeUTC,
                    updated_on_utc: now,
                    agent_id: a.agentId,
                    queue_id: q.value,
                    source: 'agatha',
                    is_active: isActive,
                    author: currentUserEmail,
                };

                if (additionalColumns) {
                    for (let key of Object.keys(additionalColumns)) {
                        agent[key] = a[additionalColumns[key]]
                    }
                }
                return agent;
            });
        }));
    };

    buildUpdateQueries = (agentQueues, now) => {
        let conditions = '';
        const {additionalColumns} = this.props;
        agentQueues.forEach(function (agent) {
            conditions += `
            {
                agent_id: {_eq: ${agent.id}},
                queue_id: {_nin: [${agent.activeQueues.join(',')}]},
            }
            `
        });

        const author = currentUserEmail;
        const deleteQuery = gql`
            mutation {
              update_agent_shift_queue(
                where: {_or: [${conditions}]},
                _set: {is_active: false, updated_on_utc: "${now}", author: "${author}"}
              ) {
                affected_rows
              }
            }
        `;

        let updateColumns = [
            'author', 'source', 'shift_id', 'start_time_utc', 'end_time_utc', 'updated_on_utc'
        ];
        if (additionalColumns) {
            updateColumns = [...updateColumns, ...Object.keys(additionalColumns)];
        }

        const upsertQuery = gql`
            mutation($agents: [agent_shift_queue_insert_input!]!) {
              insert_agent_shift_queue(
                objects: $agents,
                on_conflict: {
                  constraint: agent_shift_queue_pk
                  update_columns: [${updateColumns.join(', ')}]
                }
              ) {
                affected_rows
              }
            }
        `;
        return {deleteQuery, upsertQuery};
    };

    render() {
        const {agents, onCompleted} = this.props;
        const updatedAgentQueues = agents.map(x => {
            return {
                id: x.agentId,
                activeQueues: x.activeQueues.map(y => y.value),
            }
        });
        const now = moment.utc().format('YYYY-MM-DDTHH:mm:ss');
        const {deleteQuery, upsertQuery} = this.buildUpdateQueries(updatedAgentQueues, now);
        const updatedAgents = this.flattenAgentQueues(agents, now, true);

        return (
            <Mutation
                mutation={upsertQuery}
                onCompleted={() => {
                    if (typeof onCompleted === 'function') {
                        onCompleted();
                    }
                }}
            >
                {(upsertQueues, {loading: upsLoading, error: upsError}) => {
                    return (
                        <Mutation
                            mutation={deleteQuery}
                            onCompleted={() => {
                                upsertQueues({variables: {agents: updatedAgents}})
                            }}
                        >
                            {(deleteQueues, {loading: delLoading, error: delError}) => {
                                if (delLoading || upsLoading) return null;
                                if (delError) return <Error
                                    msg={'Failed to delete agent\'s inactive queues'}/>;
                                if (upsError) return <Error
                                    msg={'Failed to update agent\'s active queues'}/>;

                                return (
                                    <Button color="primary" onClick={deleteQueues}>
                                        Confirm
                                    </Button>
                                );
                            }}
                        </Mutation>
                    );
                }}
            </Mutation>
        );
    }
}

QueueChangesTable.propTypes = {
    agents: PropTypes.array.isRequired,
    onCompleted: PropTypes.func,
    additionalColumns: PropTypes.object,
};


export class Stats extends React.Component {

    agentControlsQuery = gql`
        query {
          selectedStartDate @client
          selectedEndDate @client
          selectedChannels @client
          selectedTiers @client
          selectedSites @client
          selectedTeams @client
        }
    `;

    getAgentsStatsQuery = () => {
        let query = `query($start: timestamp!, $end: timestamp!, $channel: [String]!, 
          $team: [String]!, $tier: [String]!, $site: [String]!) {`;

        ['agent_on_shift', 'agent_off_shift'].forEach(el => {
            query += `${el}: ${el}_aggregate (where: {
                _or: [
                  { start_time_utc: {_gte: $start _lte: $end} }
                  {
                    start_time_utc: {_lte: $start}
                    end_time_utc: {_gte: $start}
                  }
                ],
                agent: {
                  channel: {_in: $channel}
                  team: {_in: $team}
                  tier: {_in: $tier}
                  site: {_in: $site}
                }
              }) {
                aggregate {
                  count
                }
              }`;
        });

        query += '}';
        return gql(query);
    };

    render() {
        const {classes} = this.props;

        return (
            <Query query={this.agentControlsQuery}>
                {({data}) => {
                    const variables = parseAgentControlsQuery(data);
                    return (
                        <Query query={this.getAgentsStatsQuery()} variables={variables}>
                            {({loading, error, data}) => {
                                if (loading) return <Progress/>;
                                if (error) return <Error msg={'Failed to load agent counts'}/>;

                                const {agent_on_shift, agent_off_shift} = data;
                                const onShiftCount = agent_on_shift.aggregate.count;
                                const offShiftCount = agent_off_shift.aggregate.count;
                                const totalCount = onShiftCount + offShiftCount;

                                return (
                                    <div className={classes.statsWrapper}>
                                        <Typography className={classes.statWrapper}>
                                            {totalCount} agents <b>total</b>
                                            <span>, </span>
                                            {onShiftCount} <b>on</b>
                                            <span> </span>
                                            and {offShiftCount} <b>off</b> shift.
                                        </Typography>
                                    </div>
                                )
                            }}
                        </Query>
                    );
                }}
            </Query>
        )
    }
}

Stats.propTypes = {
    classes: PropTypes.object.isRequired,
    client: PropTypes.object.isRequired,
};


export class ShiftToggle extends React.Component {
    query = gql`query { isOnShift }`;

    handleClick = () => {
        const {client} = this.props;
        const {isOnShift} = client.readQuery({query: this.query});
        client.writeQuery({query: this.query, data: {isOnShift: !isOnShift}});
        this.forceUpdate();
    };

    render() {
        const {classes} = this.props;
        const {client} = this.props;
        const {isOnShift} = client.readQuery({query: this.query});
        const title = `Display Agents ${isOnShift ? 'OFF' : 'ON'} Shift`;

        return (
            <React.Fragment>
                <Tooltip title={title}>
                    <IconButton className={classes.iconButton} onClick={this.handleClick}>
                        {isOnShift ? <DomainDisabledIcon/> : <DomainIcon/>}
                    </IconButton>
                </Tooltip>
            </React.Fragment>
        );
    }
}

ShiftToggle.propTypes = {
    classes: PropTypes.object.isRequired,
    client: PropTypes.object.isRequired,
};